import java.util.List;

import java.util.stream.Collectors;
import java.util.stream.Stream;

import static java.lang.StackWalker.Option.*;

public class SWDemo
{
   public static void main(String[] args) throws Exception
   {
      // 1. Demonstrate what happens when RETAIN_CLASS_REFERENCE is missing.

      try
      {
         StackWalker sw = StackWalker.getInstance();
         sw.forEach(f -> System.out.printf("Declaring class: %s%n", 
                                           f.getDeclaringClass()));
      }
      catch (UnsupportedOperationException uoe)
      {
         System.out.printf("Unsupported operation: %s%n", uoe.getMessage());
      }
      System.out.println();

      // 2. Demonstrate what happens when RETAIN_CLASS_REFERENCE is present.

      StackWalker sw = StackWalker.getInstance(RETAIN_CLASS_REFERENCE);
      sw.forEach(f -> System.out.printf("Declaring class: %s%n", 
                                        f.getDeclaringClass()));
      System.out.println();

      // 3. Demonstrate what happens when SHOW_REFLECT_FRAMES and 
      //    SHOW_HIDDEN_FRAMES are absent.

      invokeReflectively(() -> StackWalker.getInstance()
                                          .forEach(System.out::println));
      System.out.println();

      // 4. Demonstrate what happens when SHOW_REFLECT_FRAMES is 
      //    present.

      invokeReflectively(() -> StackWalker.getInstance(SHOW_REFLECT_FRAMES)
                                          .forEach(System.out::println));
      System.out.println();

      // 5. Demonstrate what happens when SHOW_HIDDEN_FRAMES is 
      //    present.

      invokeReflectively(() -> StackWalker.getInstance(SHOW_HIDDEN_FRAMES)
                                          .forEach(System.out::println));
   }

   public static void invokeReflectively(Runnable r) throws Exception 
   {
      SWDemo.class.getMethod("run", Runnable.class).invoke(null, r);
   }

   public static void run(Runnable r) 
   {
      r.run();
   }
}